# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

import os
import sys

# Add the project root to the Python path
sys.path.insert(0, os.path.abspath(".."))

# Project information
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information

project = "LISA Artifacts"
author = "LISA Consortium"

# The full version, including alpha/beta/rc tags
release = "latest"  # Will be overridden by multiversion

# General configuration
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions = [
    "myst_parser",
    "sphinx_multiversion",
    "sphinx.ext.autodoc",
    "sphinx.ext.napoleon",
]

exclude_patterns = ["_build"]

# Options for MyST
myst_enable_extensions = [
    "colon_fence",
    "deflist",
]

# Options for HTML output
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

html_theme = "sphinx_rtd_theme"
html_show_sphinx = False

html_static_path = ["_static"]

templates_path = [
    "_templates",
]

# These paths are either relative to html_static_path
# or fully qualified paths (eg. https://...)
html_css_files = [
    "css/custom.css",
]


# Multiversion configuration
# Build versions from tags matching vX.Y.Z and main branch
# Whitelist pattern for tags (set to None to ignore all tags)
smv_tag_whitelist = r"^v.*$"

# Whitelist pattern for branches (set to None to ignore all branches)
smv_branch_whitelist = r"^.*$"

# Whitelist pattern for remotes (set to None to use local branches only)
smv_remote_whitelist = None

# Pattern for released versions
smv_released_pattern = r"^tags/v.*$"

# Format for versioned output directories inside the build directory
smv_outputdir_format = "{ref.name}"

# Determines whether remote or local git branches/tags are preferred if their output
# dirs conflict
smv_prefer_remote_refs = False



import logging
import os
from pathlib import Path
from typing import Annotated, Self

import boto3
from pydantic import BaseModel, Field
from types_boto3_s3 import S3Client

logger = logging.getLogger(__name__)


class Artifact(BaseModel):
    """Represents an artifact with a name and local path."""

    name: Annotated[str, Field(min_length=1, strict=True)]
    """The name of the artifact, usefffd as the S3 key."""

    path: Annotated[str, Field(min_length=1, strict=True)]
    """The local file path relative to the local prefix."""


class ManifestConfiguration(BaseModel):
    """Configuration for manifest operations."""

    bucket: Annotated[str, Field(min_length=1, strict=True)]
    """The S3 bucket name."""

    remote_prefix: Annotated[str, Field(strict=True)] = ""
    """Prefix for S3 keys."""

    local_prefix: Annotated[str, Field(strict=True)] = ""
    """Prefix for local paths."""


class GetManifestResult(BaseModel):
    """Result of a get operation. Currently empty."""


class StoreManifestResult(BaseModel):
    """Result of a store operation. Currently empty."""


class Manifest(BaseModel):
    """Manages a collection of artifacts for S3 operations."""

    config: Annotated[ManifestConfiguration, Field()]
    """Configuration for the manifest."""

    artifacts: Annotated[list[Artifact], Field(strict=True)]
    """List of artifacts to manage."""

    @classmethod
    def from_env(cls, artifacts: list[Artifact]) -> Self:
        """Create a Manifest from environment variables.

        Args:
            artifacts: List of artifacts to include.

        Returns:
            A new Manifest instance.

        Raises:
            KeyError: If ARTIFACTS_BUCKET is not set.

        """
        return cls(
            config=ManifestConfiguration(
                bucket=os.environ["ARTIFACTS_BUCKET"],
                remote_prefix=os.environ.get("ARTIFACTS_REMOTE_PREFIX", ""),
                local_prefix=os.environ.get("ARTIFACTS_LOCAL_PREFIX", ""),
            ),
            artifacts=artifacts,
        )

    def get(self) -> GetManifestResult:
        """Download all artifacts from S3.

        Returns:
            Result of the operation.

        """
        client = boto3.client("s3")
        for i, a in enumerate(self.artifacts, start=1):
            logger.info(
                "Downloading artifact %d/%d: %s", i, len(self.artifacts), a.name
            )
            self.get_artifact(client, a)
        return GetManifestResult()

    def store(self) -> StoreManifestResult:
        """Upload all artifacts to S3.

        Returns:
            Result of the operation.

        """
        client = boto3.client("s3")
        for i, a in enumerate(self.artifacts, start=1):
            logger.info("Storing artifact %d/%d: %s", i, len(self.artifacts), a.name)
            self.store_artifact(client, a)

        return StoreManifestResult()

    def get_artifact(self, client: S3Client, artifact: Artifact):
        """Download a single artifact from S3.

        Args:
            client: S3 client.
            artifact: The artifact to download.

        """
        artifact_key = f"{self.config.remote_prefix}{artifact.name}"
        local_path = Path(self.config.local_prefix) / artifact.path
        local_path.parent.mkdir(parents=True, exist_ok=True)
        client.download_file(
            Bucket=self.config.bucket,
            Key=artifact_key,
            Filename=str(local_path),
        )
        logger.debug("Downloaded %s to %s", artifact_key, local_path)

    def store_artifact(self, client: S3Client, artifact: Artifact):
        """Upload a single artifact to S3.

        Args:
            client: S3 client.
            artifact: The artifact to upload.

        """
        artifact_key = f"{self.config.remote_prefix}{artifact.name}"
        local_path = Path(self.config.local_prefix) / artifact.path
        client.upload_file(
            Filename=str(local_path),
            Bucket=self.config.bucket,
            Key=artifact_key,
        )
        logger.debug("Uploaded %s to %s", local_path, artifact_key)
